
# q-expansion of newform 1122.2.a.p, downloaded from the LMFDB on 08 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1122
weight = 2
poly_data = [-2, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[0, 2], 1]]

hecke_ring_character_values = None
aps_data = [[-1, 0], [-1, 0], [0, 0], [-2, 1], [1, 0], [0, -2], [1, 0], [0, -1], [2, -1], [2, 3], [-4, 1], [-6, 1], [6, -2], [-4, 1], [-6, 1], [-4, 0], [-4, -1], [0, 2], [-4, 2], [-6, -1], [-2, -3], [-14, 1], [4, -2], [2, 0], [-10, 2], [6, -4], [-12, 0], [-4, 2], [4, 0], [-10, -3], [-2, -3], [-4, 0], [-2, 6], [-12, 2], [10, 2], [-2, -1], [-6, 6], [4, -2], [0, -5], [-14, 1], [-4, -3], [6, -1], [10, -5], [18, -3], [-6, 5], [-8, -5], [0, 6], [-8, -2], [4, 6], [18, 2], [6, 4], [12, 4], [-6, 1], [4, -3], [2, 8], [-8, 6], [0, 4], [2, -1], [-8, -2], [26, 0], [0, 6], [-6, -8], [0, 1], [-22, 1], [2, 2], [-4, -6], [4, -4], [-2, -5], [20, 2], [0, 10], [-6, 2], [-12, -2], [-4, -11], [24, 2], [12, 0], [-2, 5], [-12, 0], [6, -7], [-14, 3], [6, -6], [0, 4], [14, 8], [4, 3], [-6, 0], [18, 3], [4, -9], [34, 1], [-2, 6], [2, -2], [4, 8], [-16, -7], [-12, 5], [-8, -3], [12, -10], [-36, 0], [16, -5], [-4, 0], [-14, 3], [12, 3], [-20, -8], [4, -12], [-6, -12], [-20, 6], [-14, 4], [12, 10], [14, 4], [20, -5], [-22, -8], [-10, 11], [-10, 5], [2, 1], [-12, -2], [14, 5], [4, -6], [-8, 6], [-6, -3], [20, -4], [-10, -1], [-4, 6], [36, 4], [-22, -2], [-14, 9], [18, -3], [-16, -8], [-12, -10], [2, 6], [30, 3], [2, 9], [36, -2], [0, -2], [-20, -7], [24, -9], [-28, -1], [14, 4], [-6, 0], [-2, 18], [-12, 8], [0, -16], [-20, 0], [-2, -16], [40, -4], [18, 7], [12, -3], [28, 6], [-2, -6], [-10, 9], [8, 2], [14, -4], [12, 12], [2, -3], [-4, 4], [-2, 1], [12, 8], [-24, -3], [-12, 6], [10, -5], [-54, 1], [10, 7], [-30, 8], [-30, -1], [28, 6], [14, -8], [30, 3], [12, -17], [22, -6], [-14, 5], [-28, -5], [-12, 12], [-22, -3], [4, 2], [44, 0], [-30, -8], [0, 1], [10, -8], [24, 0], [30, -1], [-4, -8], [4, 6], [2, 5], [24, -10], [-24, -14], [-20, 6], [2, -17], [6, -8], [-42, 7], [24, 4], [0, -2], [-20, 8], [-6, -3], [36, 3], [-10, 16], [44, 8], [4, 8], [-12, 8], [20, 8], [-26, -3], [6, -1], [6, -11], [14, 6], [-8, -8], [-10, -11], [-10, -9], [2, -4], [2, 12], [36, -3], [-32, -12], [36, 4], [4, -2], [42, 2], [4, -2], [-38, -9], [-32, -4], [-36, -5], [24, 3], [-16, -15], [14, -18], [-18, -7], [18, -4], [-62, 1], [-32, 12], [-36, -10], [-38, -1], [-26, 8], [28, -3], [28, 6], [-16, 6], [14, -7], [-18, -9], [-58, -1], [-20, -10], [22, 8], [-48, 2], [-30, -1], [10, 4], [-4, -12], [-24, -4], [-6, 14], [-26, 9], [-16, -8], [-22, -7], [12, 4], [-36, -11], [-16, -12], [2, -10], [-6, 8], [48, -5], [24, -15], [20, -16], [16, 3], [20, 6], [36, -2], [-34, 0], [26, -15], [46, -10], [42, 6], [60, 6], [-6, 7], [-40, -4], [4, -26], [22, -4], [6, -3], [-20, 6], [-60, 4], [-4, -4], [-6, 5], [-28, 0], [-8, -20], [-30, 6], [-12, -10], [-26, -18], [-26, 9], [4, 8], [6, -20], [46, 9], [-30, 0], [-20, -4], [48, 9], [-20, 10], [-6, 4], [-12, 10], [-8, 14], [8, 17], [-56, 1], [-8, -14], [36, 12], [6, 21], [-26, 10], [26, 3], [-12, 6], [-38, 8], [-24, -8], [-16, 16], [-22, 16], [4, -10], [-4, 24], [22, -15], [28, 4], [52, 4], [-28, 4], [36, -1], [22, 0], [-30, 2], [-2, -25], [20, -2], [-4, 16], [26, 0], [56, 11], [-30, -7], [10, 1], [24, -10], [16, 3], [20, 2], [-2, -18], [-12, 10], [8, -4], [10, 12], [24, -15], [-52, 3], [-46, 4], [38, -12], [-12, -14], [22, 20], [-6, 12], [-4, -16], [-38, -6], [-34, -8], [20, 10], [36, -12], [40, -11], [26, -11], [-72, 6], [-8, -4], [-58, -11], [12, -10], [-40, -4], [44, -6], [74, -6], [42, 0], [50, -6], [46, 7], [-6, 10], [54, -6], [14, -7], [32, 2], [-12, 24], [36, 10], [-50, 1], [12, -2], [48, 3], [0, 0], [-52, -7], [18, 16], [28, -16], [2, 13], [-20, 2], [30, -4], [14, 25], [12, 16], [6, 0], [-14, -5], [72, -6], [-46, 4], [22, 9], [20, 4], [20, 4], [26, 0], [-42, 17], [46, -1], [-54, -11], [20, 7], [-28, -2], [44, -17], [-2, 10], [30, -3], [-46, -3], [-20, -6], [-24, -15], [-30, -16], [-50, -8], [14, 24], [-32, -8], [22, 24], [18, -7], [18, -6], [-32, -2], [-42, -13], [64, 0], [22, 8], [0, -12], [34, 9], [-42, -12], [52, 14], [12, 5], [44, -4], [-6, -17], [50, 15], [-14, 17], [-2, 16], [44, 20], [6, -8], [-32, -18], [6, -26], [-38, -3], [-22, 5], [-30, 20], [70, 7], [-10, 18], [22, 20], [12, -4], [52, -2], [-50, 15], [-46, -12], [-72, 1], [28, -22], [-30, -6], [12, 14], [-16, -25], [-56, 9], [-54, -1], [-84, 0], [-16, 0], [24, 10], [58, -5], [68, 2], [14, 2], [-20, -8], [44, 0], [-26, 28], [44, -12], [12, -3]]
